/*
 * Service routine called by RPC library. Modified from a template generated by
 * rpcgen.
 */

#include <date.h>

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <arpa/inet.h>

char *get_ip_str(const struct sockaddr *sa, char *s, size_t maxlen) {
  // https://gist.github.com/jkomyno/45bee6e79451453c7bbdc22d033a282e
  switch (sa->sa_family) {
  case AF_INET:
    inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr), s, maxlen);
    break;

  case AF_INET6:
    inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr), s, maxlen);
    break;

  default:
    strncpy(s, "Unknown AF", maxlen);
    return NULL;
  }

  return s;
}

char** get_remote_date_1_svc(char **argp, struct svc_req *rqstp) {
  struct sockaddr_in *sock = svc_getcaller(
      rqstp->rq_xprt); // The approved way of getting the network address of the
                       // caller of a procedure associated with the RPC service
                       // transport handle, xprt.
  char addrInfo[100];

  get_ip_str((struct sockaddr *)sock, addrInfo, sizeof addrInfo);

  printf("%s:%d - Call received in prog %lu vers %lu proc %lu addr %s\n",
          __FILE__, __LINE__,
          rqstp->rq_prog, rqstp->rq_vers, rqstp->rq_proc, addrInfo);

  time_t systime = time(NULL);

  char timeBuf[26]; // Must be static to be returned. Size from man page
                           // for ctime_r

  ctime_r(&systime, timeBuf); // timeBuf holds the null-terminated formatted
                              // time with a trailing \n
  size_t i = strlen(timeBuf);
  if (i > 0 && i < sizeof timeBuf && timeBuf[i - 1] == '\n') {
    timeBuf[i - 1] = '\0';
  }
  else {
    fprintf(stderr, "%s:%d ctime_r() failed\n", __FILE__, __LINE__);
    exit(EXIT_FAILURE);
  }

  /* TI-RPC allows a single parameter to be passed from client to server. If
   * more than one parameter is required, the components can be combined into a
   * structure, that is counted as a single element. Information passed from
   * server to client is passed as the function's return value. Information
   * cannot be passed back from server to client through the parameter list.
   * https://docs.oracle.com/cd/E19455-01/805-7224/6j6q44cgg/index.html
   */

  printf("%s:%d - received %lu bytes \"%s\"\n", __FILE__, __LINE__,
         strlen(*argp), *argp);
  size_t returnLen = strlen(timeBuf) + 1; // + 1 to make room for null
  static char** r = 0;
  if (r && *r != NULL) {
    free (*r);
  }
  r = (char**)malloc(sizeof(char*));  // allocate a pointer
  *r = (char*)malloc(returnLen);
  strcpy (*r, timeBuf);
  printf("%s:%d - returning %lu bytes \"%s\"\n", __FILE__, __LINE__,
         strlen(*r), *r);
  return r;
}
